/*
   Copyright (C) 2002 Jason Stechschulte <stech@enormousplow.com>

	This program is free software; you can redistribute it and/or modify it
	under the terms of the GNU General Public License.  This program is
	distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/

#include "fred.h"

#include "window.h"
#include "images.h"
#include "scoring.h"
#include "dialogs.h"
#include "plane.h"
#include "diver.h"

SDL_Surface* render(TTF_Font *font, string &message, SDL_Color &color) {
	SDL_Surface *temp;
	temp = TTF_RenderText_Solid(font, message.c_str(), color);
	if(temp == NULL) {
		cerr << "Couldn't render text: " << SDL_GetError() << endl;
		TTF_CloseFont(font);
		throw;
	}
	return temp;
}

int main(int argc, char* argv[]) {

	// This might be the actual data directory
	const string DATADIR = "/usr/share/games/fred/data/";

	// Uncomment if compiling for windows
	//const string DATADIR = "data/";

	// These should be obvious.  If not, seek help
	int paused = 0;
	int done = 0;
	int numSprites = 0;
	int totalSprites = 0;
	int random;

	int lost = 0;

	// Seed with current time
	srand(time(NULL));
	// Let it run a while to help with randomness
	for(int i = 0, random = rand(); i < 50; i++, random = rand()) {}
	// Now decide when Fred will jump
	random = rand() % 250 + 10;

	// These are used for frame-rate independent movement
	Uint32 curTime = 0;
	Uint32 lastUpdate = 0;
	float lag = 0;

	// Create game window
	Window window;

	// Load the images
	Images images(DATADIR);

	// Show startup screen
	if(!begin(window, images, DATADIR)) {
      return 0;
	}

	// Loading sprites
	Plane plane(images, -50, 10);
	numSprites++;
	Diver *fred = new Diver(images, -47, 10, random);
	numSprites++;
	totalSprites = numSprites;

	// Keep score
	Scoring score(DATADIR);

	// Start frame-rate independence
	curTime = SDL_GetTicks();
	lastUpdate = curTime;

	if(DEBUG) cout << "Beginning game" << endl;

	// Game loop
	while(!done) {
		// I think this is used to capture an event.  Not sure though
		SDL_Event event;
		// By golly, I was right!
		while(SDL_PollEvent(&event)) {
			if(event.type == SDL_QUIT) {
				// If they want to quit, we should quit
				done = 1;
			}
			// Did they click on Fred?
			else if(event.type == SDL_MOUSEBUTTONDOWN && !paused) {
				if(event.button.button == SDL_BUTTON_LEFT) {
					// Well let's find out by trying to open his chute
					for(Diver *thisFred = fred; thisFred != NULL; thisFred = thisFred->getNext()) {
						if(thisFred->openChute(event.button.x, event.button.y)) {
							score.add(event.button.y / 10);
							break;
						}
					}
				}
			}
			// Maybe they are pressing keys
			else if(event.type == SDL_KEYDOWN) {
				if(event.key.keysym.sym == SDLK_q) {
					// If they want to quit, we should quit
					done = 1;
				}
				else if(event.key.keysym.sym == SDLK_p) {
					// They are trying to pause or unpause the game
					paused = 1;
				}
			}
		}
		// Get current time
		curTime = SDL_GetTicks();
		// Determine lag
		lag = float(curTime - lastUpdate) * .001;
		// Make sure this gets updated as soon as possible to make it most accurate
		lastUpdate = curTime;

		// Draw a blue background
	  	window.drawBG();

		// <sarcasm>I have no idea what this does</sarcasm>
		if(!paused) {

			// See if Fred is jumping
			for(Diver *thisFred = fred; thisFred != NULL; thisFred = thisFred->getNext()) {
				thisFred->attemptJump(int(plane.getX()));
			}

			// Move the airplane
			if(plane.isVisible()) {
				if(!plane.move(window, lag)) {
					// If here, the plane has moved off screen
					plane.makeInvisible();
					numSprites--;
				}
			}
			// Move the Freds
			for(Diver *thisFred = fred; thisFred != NULL; thisFred = thisFred->getNext()) {
				if(!thisFred->move(window, plane, lag)) {
					thisFred->makeInvisible();
					numSprites--;
				}
			}
		}
		if(numSprites > 0) {
			// If we still have sprites on screen, draw stuff
			plane.draw(window);
			for(Diver *thisFred = fred; thisFred != NULL; thisFred = thisFred->getNext()) {
				thisFred->draw(window);
			}
			score.draw(window);
			// This is here so the pause image is in front of everything else
			if(paused) {
				window.pause(images);
				// Unpause game
				paused = 0;
				// Make it seem as if no time has passed while we were paused
				curTime = SDL_GetTicks();
				lastUpdate = curTime;
			}
			else {
				// Draws the buffer to the screen
				window.drawScene();
			}
		}
		else {
			for(Diver *thisFred = fred; thisFred != NULL; thisFred = thisFred->getNext()) {
				lost += !thisFred->chuteIsOpen();
			}
			// Otherwise, show the end stuff
			if(lost) {
				done = showEnd(window, DATADIR);
			}
			else {
				// Give 50 bonus points for saving all Freds.
				score.add(50);
			}
			// Wants to play again, lets reinitialize
			if(done == 0) {
				// Reset sprites
				plane.reset();
				if(lost) {
					for(Diver *thisFred = fred; thisFred != NULL; thisFred = thisFred = fred) {
						fred = thisFred->getNext();
						delete thisFred;
					}
					totalSprites = 1;
					random = rand() % 250 + 10;
					fred = new Diver(images, -47, 10, random);
					totalSprites++;
					lost = 0;
					score.reset();
				}
				else {
					for(Diver *thisFred = fred; thisFred != NULL; thisFred = thisFred->getNext()) {
						// Get a new random number
						random = rand() % 250 + 10;
						thisFred->reset(random);
					}
					// Add a new Fred
					random = rand() % 250 + 10;
					Diver *newFred = new Diver(images, -47, 10, random);
					newFred->setNext(fred);
					fred = newFred;
					totalSprites++;
				}
				// Reset the number of sprites
				numSprites = totalSprites;
				// Reset the time stuff again
				curTime = SDL_GetTicks();
				lag = float(curTime - lastUpdate) * .001;
				lastUpdate = curTime;
			}
		}
	}

	// Play nice
	return 0;
}
